/*** RemoteAccessStatus Z-Way HA module *******************************************

Version: 1.0.0
(c) Z-Wave.Me, 2017
-----------------------------------------------------------------------------
Author: Karsten Reichel <kar@zwave.eu>
Description:
    This module will report the find.z-wave.me status

******************************************************************************/

// ----------------------------------------------------------------------------
// --- Class definition, inheritance and setup
// ----------------------------------------------------------------------------


function RemoteAccessStatus (id, controller) {
    // Call superconstructor first (AutomationModule)
    RemoteAccessStatus.super_.call(this, id, controller);
}

inherits(RemoteAccessStatus, AutomationModule);

_module = RemoteAccessStatus;

// ----------------------------------------------------------------------------
// --- Module instance initialized
// ----------------------------------------------------------------------------

RemoteAccessStatus.prototype.init = function (config) {
    RemoteAccessStatus.super_.prototype.init.call(this, config);

        var vDevId = 'RemoteAccessStatus_' + this.id;
        
        var self = this; 

        this.vDev = this.controller.devices.create({
            deviceId: vDevId,
            defaults: {
                deviceType: 'sensorBinary',
                metrics: {
                    icon: 'switch',
                    level: 'off', 
                    title: 'Remote Access Status',
                    failureCount: 0
                }
            },
            overlay: {
                deviceType: 'sensorBinary'
            },
            handler: function(command) {

                device = this;
            
                if (command == 'update') {
                    http.request({
                        method: 'GET',
                        url: 'https://find.z-wave.me/zboxweb',
                        async: true,
                        success: function(response) {
                            var ip = system("hostname -I | cut -d ' ' -f1")[1].replace(/[\s\n]/g, '');
                            if (response.data.indexOf(ip) > -1) {
                                device.set('metrics:level', 'on');
                                device.set('metrics:failureCount',0);
                            }
                            else {
                                if (device.get('metrics:failureCount') >= 5)
                                {
                                    device.set('metrics:level', 'off');
                                } 
                                else {
                                    device.set('metrics:failureCount',device.get('metrics:failureCount')+1);
                                }                               
                            }
                        },
                        error: function(response) {
                            if (device.get('metrics:failureCount') >= 5)
                            {
                                device.set('metrics:level', 'off');
                            }
                            else {
                                device.set('metrics:failureCount',device.get('metrics:failureCount')+1);
                            }
                        } 
                    });
                }
            },    
            moduleId: this.id
        });

        var p = Math.round(this.config.interval);
        var m = (p < 60) ? [0, 59, p] : 0;
        var h = p >= 24*60 ? 0 : (p/60 >=1 ? [0, 23, Math.round(p/60)] : null);
        var wd = p/24/60 >=1 ? [0, 6, Math.round(p/24/60)] : null;
         
        this.controller.emit('cron.addTask', 'RemoteAccessStatus.poll', {
            minute: m,
            hour: h,
            weekDay: wd,
            day: null,
            month: null
        });

        this.onPoll = function () {
            self.vDev.performCommand('update');
        };        

        this.controller.on('RemoteAccessStatus.poll', self.onPoll);
};

RemoteAccessStatus.prototype.stop = function () {

    if (this.vDev) {
        this.controller.devices.remove(this.vDev.id);
        this.vDev = null;
    }

    this.controller.off('RemoteAccessStatus.poll', this.onPoll);
    this.controller.emit('cron.removeTask', 'RemoteAccessStatus.poll');

    RemoteAccessStatus.super_.prototype.stop.call(this);
};
